@@ -107,6 +107,7 @@ lookup_binding(FILE *f, char *map_wwid, char **map_alias, char *prefix)
 
 	*map_alias = NULL;
 
+	rewind(f);
 	while (fgets(buf, LINE_MAX, f)) {
 		char *c, *alias, *wwid;
 		int curr_id;
@@ -280,6 +281,13 @@ use_existing_alias (char *wwid, char *file, char *alias_old,
 		goto out;
 	}
 
+	id = lookup_binding(f, wwid, &alias, NULL);
+	if (alias) {
+		condlog(3, "Use existing binding [%s] for WWID [%s]",
+			alias, wwid);
+		goto out;
+	}
+
 	/* allocate the existing alias in the bindings file */
 	id = scan_devname(alias_old, prefix);
 	if (id <= 0)
