@@ -7,6 +7,7 @@
 #include "config.h"
 #include "pgpolicies.h"
 #include "prio.h"
+#include "hwtable.h"
 
 /*
  * Tuning suggestions on these parameters should go to
@@ -37,11 +38,10 @@
 	 *
 	 * COMPANY_NAME
 	 *
-	 * Maintainer : XXX
-	 * Mail : XXX
+	 * Maintainer: NAME <email>
 	 */
 	{
-		/* If product-ID is different from marketing name add a comment */
+		/* Product Name */
 		.vendor        = "VENDOR",
 		.product       = "PRODUCT",
 		.revision      = "REVISION",
@@ -72,17 +72,33 @@
 		.delay_wait_checks = DELAY_CHECKS_OFF,
 		.skip_kpartx   = SKIP_KPARTX_OFF,
 		.max_sectors_kb = MAX_SECTORS_KB_UNDEF,
+		.ghost_delay   = GHOST_DELAY_OFF,
 	},
 #endif
 
 static struct hwentry default_hw[] = {
 	/*
-	 * Apple
+	 * Generic NVMe devices
 	 *
-	 * Maintainer : Shyam Sundar
-	 * Mail : g.shyamsundar@yahoo.co.in
+	 * Due to the parsing logic in find_hwe(), generic entries
+	 * have to be put on top of this list, and more specific ones
+	 * below.
 	 */
 	{
+		/* Generic NVMe */
+		.vendor        = "NVME",
+		.product       = ".*",
+		.uid_attribute = DEFAULT_NVME_UID_ATTRIBUTE,
+		.checker_name  = NONE,
+		.retain_hwhandler = RETAIN_HWHANDLER_OFF,
+	},
+	/*
+	 * Apple
+	 *
+	 * Maintainer: Shyam Sundar <g.shyamsundar@yahoo.co.in>
+	 */
+	{
+		/* Xserve RAID */
 		.vendor        = "APPLE",
 		.product       = "Xserve RAID",
 		.pgpolicy      = MULTIBUS,
@@ -91,6 +107,7 @@ static struct hwentry default_hw[] = {
 	 * HPE
 	 */
 	{
+		/* 3PAR / Primera */
 		.vendor        = "3PARdata",
 		.product       = "VV",
 		.pgpolicy      = GROUP_BY_PRIO,
@@ -163,7 +180,7 @@ static struct hwentry default_hw[] = {
 		.prio_name     = PRIO_ALUA,
 	},
 	{
-		/* MSA 1040, 2040 and 2050 families */
+		/* MSA 1040, 1050, 2040 and 2050 families */
 		.vendor        = "HP",
 		.product       = "MSA [12]0[45]0 SA[NS]",
 		.pgpolicy      = GROUP_BY_PRIO,
@@ -206,21 +223,76 @@ static struct hwentry default_hw[] = {
 		.no_path_retry = 18,
 		.prio_name     = PRIO_ALUA,
 	},
+	{
+		/* Nimble Storage */
+		.vendor        = "Nimble",
+		.product       = "Server",
+		.hwhandler     = "1 alua",
+		.pgpolicy      = GROUP_BY_PRIO,
+		.pgfailback    = -FAILBACK_IMMEDIATE,
+		.prio_name     = PRIO_ALUA,
+		.no_path_retry = NO_PATH_RETRY_QUEUE,
+	},
+		/* SGI */
+	{
+		/* Total Performance 9100 */
+		.vendor        = "SGI",
+		.product       = "TP9100",
+		.pgpolicy      = MULTIBUS,
+	},
+	{
+		/* Total Performance family */
+		.vendor        = "SGI",
+		.product       = "TP9[3457]00",
+		.bl_product    = "Universal Xport",
+		.pgpolicy      = GROUP_BY_PRIO,
+		.checker_name  = RDAC,
+		.features      = "2 pg_init_retries 50",
+		.hwhandler     = "1 rdac",
+		.prio_name     = PRIO_RDAC,
+		.pgfailback    = -FAILBACK_IMMEDIATE,
+		.no_path_retry = 30,
+	},
+	{
+		/* (RDAC) InfiniteStorage */
+		.vendor        = "SGI",
+		.product       = "IS",
+		.bl_product    = "Universal Xport",
+		.pgpolicy      = GROUP_BY_PRIO,
+		.checker_name  = RDAC,
+		.features      = "2 pg_init_retries 50",
+		.hwhandler     = "1 rdac",
+		.prio_name     = PRIO_RDAC,
+		.pgfailback    = -FAILBACK_IMMEDIATE,
+		.no_path_retry = 30,
+	},
+	{
+		/* (DDN) InfiniteStorage */
+		.vendor        = "SGI",
+		.product       = "^DD[46]A-",
+		.pgpolicy      = GROUP_BY_PRIO,
+		.pgfailback    = -FAILBACK_IMMEDIATE,
+		.prio_name     = PRIO_ALUA,
+		.no_path_retry = 30,
+	},
 	/*
 	 * DataDirect Networks
 	 */
 	{
+		/* SAN DataDirector */
 		.vendor        = "DDN",
 		.product       = "SAN DataDirector",
 		.pgpolicy      = MULTIBUS,
 	},
 	{
+		/* EF3010 */
 		.vendor        = "DDN",
 		.product       = "^EF3010",
 		.pgpolicy      = MULTIBUS,
 		.no_path_retry = 30,
 	},
 	{
+		/* EF3015 / S2A and SFA families */
 		.vendor        = "DDN",
 		.product       = "^(EF3015|S2A|SFA)",
 		.pgpolicy      = GROUP_BY_PRIO,
@@ -228,18 +300,39 @@ static struct hwentry default_hw[] = {
 		.prio_name     = PRIO_ALUA,
 		.no_path_retry = 30,
 	},
+	{
+		/*
+		 * Nexenta COMSTAR
+		 *
+		 * Maintainer: Yacine Kheddache <yacine@alyseo.com>
+		 */
+		.vendor        = "NEXENTA",
+		.product       = "COMSTAR",
+		.pgpolicy      = GROUP_BY_SERIAL,
+		.no_path_retry = 30,
+	},
+	{
+		/* Tegile IntelliFlash */
+		.vendor        = "TEGILE",
+		.product       = "(ZEBI-(FC|ISCSI)|INTELLIFLASH)",
+		.hwhandler     = "1 alua",
+		.pgpolicy      = GROUP_BY_PRIO,
+		.pgfailback    = -FAILBACK_IMMEDIATE,
+		.prio_name     = PRIO_ALUA,
+		.no_path_retry = 10,
+	},
 	/*
 	 * Dell EMC
 	 */
 	{
-		/* Symmetrix / DMX / VMAX */
+		/* Symmetrix / DMX / VMAX / PowerMax */
 		.vendor        = "EMC",
 		.product       = "SYMMETRIX",
 		.pgpolicy      = MULTIBUS,
 		.no_path_retry = 6,
 	},
 	{
-		/* DGC CLARiiON CX/AX / EMC VNX and Unity */
+		/* DGC CLARiiON CX/AX / VNX and Unity */
 		.vendor        = "^DGC",
 		.product       = "^(RAID|DISK|VRAID)",
 		.bl_product    = "LUNZ",
@@ -259,16 +352,16 @@ static struct hwentry default_hw[] = {
 		.no_path_retry = 5,
 	},
 	{
+		/* XtremIO */
 		.vendor        = "XtremIO",
 		.product       = "XtremApp",
 		.pgpolicy      = MULTIBUS,
 	},
 	{
 		/*
-		 * Dell SC Series, formerly Compellent
+		 * SC Series, formerly Compellent
 		 *
-		 * Maintainer : Sean McGinnis
-		 * Mail : sean_mcginnis@dell.com
+		 * Maintainer: Sean McGinnis <sean_mcginnis@dell.com>
 		 */
 		.vendor        = "COMPELNT",
 		.product       = "Compellent Vol",
@@ -288,15 +381,23 @@ static struct hwentry default_hw[] = {
 		.pgfailback    = -FAILBACK_IMMEDIATE,
 		.no_path_retry = 30,
 	},
+	{
+		/* EMC PowerMax NVMe */
+		.vendor        = "NVME",
+		.product       = "^EMC PowerMax_",
+		.pgpolicy      = MULTIBUS,
+	},
 	/*
 	 * Fujitsu
 	 */
 	{
+		/* CentricStor Virtual Tape */
 		.vendor        = "FSC",
 		.product       = "CentricStor",
 		.pgpolicy      = GROUP_BY_SERIAL,
 	},
 	{
+		/* ETERNUS family */
 		.vendor        = "FUJITSU",
 		.product       = "ETERNUS_DX(H|L|M|400|8000)",
 		.pgpolicy      = GROUP_BY_PRIO,
@@ -311,6 +412,7 @@ static struct hwentry default_hw[] = {
 		.pgpolicy      = MULTIBUS,
 	},
 	{
+		/* ETERNUS 2000, 3000 and 4000 */
 		.vendor        = "FUJITSU",
 		.product       = "E[234]000",
 		.pgpolicy      = GROUP_BY_PRIO,
@@ -319,39 +421,45 @@ static struct hwentry default_hw[] = {
 		.prio_name     = PRIO_ALUA,
 	},
 	{
+		/* ETERNUS 6000 and 8000 */
 		.vendor        = "FUJITSU",
 		.product       = "E[68]000",
 		.pgpolicy      = MULTIBUS,
 		.no_path_retry = 10,
 	},
 	/*
-	 * Hitachi
+	 * Hitachi Vantara
 	 *
-	 * Maintainer : Matthias Rudolph
-	 * Mail : matthias.rudolph@hds.com
+	 * Maintainer: Matthias Rudolph <Matthias.Rudolph@hitachivantara.com>
 	 */
 	{
-		/* USP-V, HUS VM, VSP, VSP G1000 and VSP GX00 families */
+		/* USP-V, HUS VM, VSP, VSP G1X00 and VSP GX00 families / HP XP */
 		.vendor        = "(HITACHI|HP)",
 		.product       = "^OPEN-",
 		.pgpolicy      = MULTIBUS,
 	},
 	{
-		/* AMS 2000 and HUS 100 families */
-		.vendor        = "(HITACHI|HP)",
+		/* AMS other than AMS 2000 */
+		.vendor        = "HITACHI",
 		.product       = "^DF",
 		.no_path_retry = NO_PATH_RETRY_QUEUE,
 		.pgpolicy      = GROUP_BY_PRIO,
 		.pgfailback    = -FAILBACK_IMMEDIATE,
 		.prio_name     = PRIO_HDS,
 	},
+	{
+		/* AMS 2000 and HUS 100 families */
+		.vendor        = "HITACHI",
+		.product       = "^DF600F",
+		.pgpolicy      = MULTIBUS,
+	},
 	/*
 	 * IBM
 	 *
-	 * Maintainer : Hannes Reinecke
-	 * Mail : hare@suse.de
+	 * Maintainer: Hannes Reinecke <hare@suse.de>
 	 */
 	{
+		/* ProFibre 4000R */
 		.vendor        = "IBM",
 		.product       = "ProFibre 4000R",
 		.pgpolicy      = MULTIBUS,
@@ -500,7 +608,7 @@ static struct hwentry default_hw[] = {
 		.no_path_retry = 30,
 	},
 	{
-		/* Enterprise Storage Server / Shark family */
+		/* Enterprise Storage Server(ESS) / Shark family */
 		.vendor        = "IBM",
 		.product       = "^2105",
 		.no_path_retry = NO_PATH_RETRY_QUEUE,
@@ -523,7 +631,7 @@ static struct hwentry default_hw[] = {
 		.pgpolicy      = MULTIBUS,
 	},
 	{
-		/* Storwize family / SAN Volume Controller / Flex System V7000 / FlashSystem V840/V9000 */
+		/* Storwize family / SAN Volume Controller / Flex System V7000 / FlashSystem V840/V9000/9100 */
 		.vendor        = "IBM",
 		.product       = "^2145",
 		.no_path_retry = NO_PATH_RETRY_QUEUE,
@@ -532,20 +640,24 @@ static struct hwentry default_hw[] = {
 		.prio_name     = PRIO_ALUA,
 	},
 	{
+		/* PAV DASD ECKD */
 		.vendor        = "IBM",
 		.product       = "S/390 DASD ECKD",
 		.bl_product    = "S/390",
 		.uid_attribute = "ID_UID",
 		.no_path_retry = NO_PATH_RETRY_QUEUE,
 		.pgpolicy      = MULTIBUS,
+		.checker_name  = DIRECTIO,
 	},
 	{
+		/* PAV DASD FBA */
 		.vendor        = "IBM",
 		.product       = "S/390 DASD FBA",
 		.bl_product    = "S/390",
 		.uid_attribute = "ID_UID",
 		.no_path_retry = NO_PATH_RETRY_QUEUE,
 		.pgpolicy      = MULTIBUS,
+		.checker_name  = DIRECTIO,
 	},
 	{
 		/* Power RAID */
@@ -580,7 +692,7 @@ static struct hwentry default_hw[] = {
 		.pgpolicy      = MULTIBUS,
 	},
 	{
-		/* DDN */
+		/* (DDN) DCS9900, SONAS 2851-DR1 */
 		.vendor        = "IBM",
 		.product       = "^(DCS9900|2851)",
 		.pgpolicy      = GROUP_BY_PRIO,
@@ -591,10 +703,10 @@ static struct hwentry default_hw[] = {
 		/*
 		 * IBM Power Virtual SCSI Devices
 		 *
-		 * Maintainer : Brian King
-		 * Mail : brking@linux.vnet.ibm.com
+		 * Maintainer: Brian King <brking@linux.vnet.ibm.com>
 		 */
 	{
+		/* AIX VDASD */
 		.vendor        = "AIX",
 		.product       = "VDASD",
 		.pgpolicy      = MULTIBUS,
@@ -607,6 +719,7 @@ static struct hwentry default_hw[] = {
 		.no_path_retry = (300 / DEFAULT_CHECKINT),
 	},
 	{
+		/* AIX NVDISK */
 		.vendor        = "AIX",
 		.product       = "NVDISK",
 		.hwhandler     = "1 alua",
@@ -615,6 +728,26 @@ static struct hwentry default_hw[] = {
 		.no_path_retry = (300 / DEFAULT_CHECKINT),
 		.prio_name     = PRIO_ALUA,
 	},
+        /*
+         * Lenovo
+         */
+        {
+                /*
+		 * DE Series
+		 *
+		 * Maintainer: NetApp RDAC team <ng-eseries-upstream-maintainers@netapp.com>
+		 */
+                .vendor        = "LENOVO",
+                .product       = "DE_Series",
+                .bl_product    = "Universal Xport",
+                .pgpolicy      = GROUP_BY_PRIO,
+                .checker_name  = RDAC,
+                .features      = "2 pg_init_retries 50",
+                .hwhandler     = "1 rdac",
+                .prio_name     = PRIO_RDAC,
+                .pgfailback    = -FAILBACK_IMMEDIATE,
+                .no_path_retry = 30,
+        },
 	/*
 	 * NetApp
 	 */
@@ -622,8 +755,7 @@ static struct hwentry default_hw[] = {
 		/*
 		 * ONTAP family
 		 *
-		 * Maintainer : Martin George
-		 * Mail : marting@netapp.com
+		 * Maintainer: Martin George <marting@netapp.com>
 		 */
 		.vendor        = "NETAPP",
 		.product       = "LUN",
@@ -634,13 +766,13 @@ static struct hwentry default_hw[] = {
 		.flush_on_last_del = FLUSH_ENABLED,
 		.dev_loss      = MAX_DEV_LOSS_TMO,
 		.prio_name     = PRIO_ONTAP,
+		.user_friendly_names = USER_FRIENDLY_NAMES_OFF,
 	},
 	{
 		/*
 		 * SANtricity(RDAC) family
 		 *
-		 * Maintainer : Sean Stewart
-		 * Mail : sean.stewart@netapp.com
+		 * Maintainer: NetApp RDAC team <ng-eseries-upstream-maintainers@netapp.com>
 		 */
 		.vendor        = "(NETAPP|LSI|ENGENIO)",
 		.product       = "INF-01-00",
@@ -657,68 +789,23 @@ static struct hwentry default_hw[] = {
 		/*
 		 * SolidFir family
 		 *
-		 * Maintainer : PJ Waskiewicz
-		 * Mail : pj.waskiewicz@netapp.com
+		 * Maintainer: PJ Waskiewicz <pj.waskiewicz@netapp.com>
 		 */
 		.vendor        = "SolidFir",
 		.product       = "SSD SAN",
 		.pgpolicy      = MULTIBUS,
 		.no_path_retry = 24,
 	},
-	/*
-	 * Nexenta
-	 *
-	 * Maintainer : Yacine Kheddache
-	 * Mail : yacine@alyseo.com
-	 */
 	{
-		.vendor        = "NEXENTA",
-		.product       = "COMSTAR",
-		.pgpolicy      = GROUP_BY_SERIAL,
-		.no_path_retry = 30,
-	},
-	/*
-	 * SGI
-	 */
-	{
-		.vendor        = "SGI",
-		.product       = "TP9100",
+		/*
+		 * NVMe-FC namespace devices: MULTIBUS, queueing preferred
+		 *
+		 * The hwtable is searched backwards, so place this after "Generic NVMe"
+		 */
+		.vendor	       = "NVME",
+		.product       = "^NetApp ONTAP Controller",
 		.pgpolicy      = MULTIBUS,
-	},
-	{
-		/* Total Performance family */
-		.vendor        = "SGI",
-		.product       = "TP9[3457]00",
-		.bl_product    = "Universal Xport",
-		.pgpolicy      = GROUP_BY_PRIO,
-		.checker_name  = RDAC,
-		.features      = "2 pg_init_retries 50",
-		.hwhandler     = "1 rdac",
-		.prio_name     = PRIO_RDAC,
-		.pgfailback    = -FAILBACK_IMMEDIATE,
-		.no_path_retry = 30,
-	},
-	{
-		/* InfiniteStorage family */
-		.vendor        = "SGI",
-		.product       = "IS",
-		.bl_product    = "Universal Xport",
-		.pgpolicy      = GROUP_BY_PRIO,
-		.checker_name  = RDAC,
-		.features      = "2 pg_init_retries 50",
-		.hwhandler     = "1 rdac",
-		.prio_name     = PRIO_RDAC,
-		.pgfailback    = -FAILBACK_IMMEDIATE,
-		.no_path_retry = 30,
-	},
-	{
-		/* DDN */
-		.vendor        = "SGI",
-		.product       = "^DD[46]A-",
-		.pgpolicy      = GROUP_BY_PRIO,
-		.pgfailback    = -FAILBACK_IMMEDIATE,
-		.prio_name     = PRIO_ALUA,
-		.no_path_retry = 30,
+		.no_path_retry = NO_PATH_RETRY_QUEUE,
 	},
 	/*
 	 * NEC
@@ -738,10 +825,10 @@ static struct hwentry default_hw[] = {
 		/*
 		 * Pillar Data / Oracle FS
 		 *
-		 * Maintainer : Srinivasan Ramani
-		 * Mail : srinivas.ramani@oracle.com
+		 * Maintainer: Srinivasan Ramani <srinivas.ramani@oracle.com>
 		 */
 	{
+		/* Axiom */
 		.vendor        = "^Pillar",
 		.product       = "^Axiom",
 		.pgpolicy      = GROUP_BY_PRIO,
@@ -749,6 +836,7 @@ static struct hwentry default_hw[] = {
 		.prio_name     = PRIO_ALUA,
 	},
 	{
+		/* FS */
 		.vendor        = "^Oracle",
 		.product       = "^Oracle FS",
 		.pgpolicy      = GROUP_BY_PRIO,
@@ -796,12 +884,13 @@ static struct hwentry default_hw[] = {
 		.no_path_retry = 30,
 	},
 	{
-		/* (Dot Hill) 3310, 3320, 3510 and 3511 */
+		/* (Dot Hill) 3120, 3310, 3320, 3510 and 3511 */
 		.vendor        = "SUN",
 		.product       = "StorEdge 3",
 		.pgpolicy      = MULTIBUS,
 	},
 	{
+		/* 6580 and 6780 */
 		.vendor        = "SUN",
 		.product       = "STK6580_6780",
 		.bl_product    = "Universal Xport",
@@ -840,6 +929,7 @@ static struct hwentry default_hw[] = {
 		.no_path_retry = 30,
 	},
 	{
+		/* 6180 */
 		.vendor        = "SUN",
 		.product       = "SUN_6180",
 		.bl_product    = "Universal Xport",
@@ -851,6 +941,19 @@ static struct hwentry default_hw[] = {
 		.pgfailback    = -FAILBACK_IMMEDIATE,
 		.no_path_retry = 30,
 	},
+	{
+		/* ArrayStorage */
+		.vendor        = "SUN",
+		.product       = "ArrayStorage",
+		.bl_product    = "Universal Xport",
+		.pgpolicy      = GROUP_BY_PRIO,
+		.checker_name  = RDAC,
+		.features      = "2 pg_init_retries 50",
+		.hwhandler     = "1 rdac",
+		.prio_name     = PRIO_RDAC,
+		.pgfailback    = -FAILBACK_IMMEDIATE,
+		.no_path_retry = 30,
+	},
 	{
 		/* ZFS Storage Appliances */
 		.vendor        = "SUN",
@@ -863,16 +966,17 @@ static struct hwentry default_hw[] = {
 	/*
 	 * Pivot3
 	 *
-	 * Maintainer : Bart Brooks
-	 * Mail : bartb@pivot3.com
+	 * Maintainer: Bart Brooks <bartb@pivot3.com>
 	 */
 	{
+		/* Raige */
 		.vendor        = "PIVOT3",
 		.product       = "RAIGE VOLUME",
 		.no_path_retry = NO_PATH_RETRY_QUEUE,
 		.pgpolicy      = MULTIBUS,
 	},
 	{
+		/* NexGen / vSTAC */
 		.vendor        = "(NexGen|Pivot3)",
 		.product       = "(TierStore|vSTAC)",
 		.pgpolicy      = GROUP_BY_PRIO,
@@ -884,6 +988,7 @@ static struct hwentry default_hw[] = {
 	 * Intel
 	 */
 	{
+		/* Multi-Flex */
 		.vendor        = "(Intel|INTEL)",
 		.product       = "Multi-Flex",
 		.bl_product    = "VTrak V-LUN",
@@ -897,6 +1002,7 @@ static struct hwentry default_hw[] = {
 	 * Linux-IO Target
 	 */
 	{
+		/* Linux-IO Target */
 		.vendor        = "(LIO-ORG|SUSE)",
 		.product       = "RBD",
 		.hwhandler     = "1 alua",
@@ -909,6 +1015,7 @@ static struct hwentry default_hw[] = {
 	 * DataCore
 	 */
 	{
+		/* SANmelody */
 		.vendor        = "DataCore",
 		.product       = "SANmelody",
 		.pgpolicy      = GROUP_BY_PRIO,
@@ -929,6 +1036,7 @@ static struct hwentry default_hw[] = {
 	 * Pure Storage
 	 */
 	{
+		/* FlashArray */
 		.vendor        = "PURE",
 		.product       = "FlashArray",
 		.pgpolicy      = MULTIBUS,
@@ -943,22 +1051,11 @@ static struct hwentry default_hw[] = {
 		.pgpolicy      = GROUP_BY_PRIO,
 		.prio_name     = PRIO_ALUA,
 	},
-	/*
-	 * Red Hat
-	 *
-	 * Maintainer: Mike Christie
-	 * Mail: mchristi@redhat.com
-	 */
-	{
-		.vendor        = "Ceph",
-		.product       = "RBD",
-		.checker_name  = RBD,
-		.deferred_remove = DEFERRED_REMOVE_ON,
-	},
 	/*
 	 * Kove
 	 */
 	{
+		/* XPD */
 		.vendor        = "KOVE",
 		.product       = "XPD",
 		.pgpolicy      = MULTIBUS,
@@ -966,10 +1063,10 @@ static struct hwentry default_hw[] = {
 	/*
 	 * Infinidat
 	 *
-	 * Maintainer: Arnon Yaari
-	 * Mail: arnony@infinidat.com
+	 * Maintainer: Arnon Yaari <arnony@infinidat.com>
 	 */
 	{
+		/* InfiniBox */
 		.vendor        = "NFINIDAT",
 		.product       = "InfiniBox",
 		.pgpolicy      = GROUP_BY_PRIO,
@@ -984,38 +1081,15 @@ static struct hwentry default_hw[] = {
 		.fast_io_fail  = 15,
 		.dev_loss      = 15,
 	},
-	/*
-	 * Nimble Storage
-	 */
-	{
-		.vendor        = "Nimble",
-		.product       = "Server",
-		.hwhandler     = "1 alua",
-		.pgpolicy      = GROUP_BY_PRIO,
-		.pgfailback    = -FAILBACK_IMMEDIATE,
-		.prio_name     = PRIO_ALUA,
-		.no_path_retry = NO_PATH_RETRY_QUEUE,
-	},
 	/*
 	 * Kaminario
 	 */
 	{
+		/* K2 */
 		.vendor        = "KMNRIO",
 		.product       = "K2",
 		.pgpolicy      = MULTIBUS,
 	},
-	/*
-	 * Tegile Systems
-	 */
-	{
-		.vendor        = "TEGILE",
-		.product       = "(ZEBI-(FC|ISCSI)|INTELLIFLASH)",
-		.hwhandler     = "1 alua",
-		.pgpolicy      = GROUP_BY_PRIO,
-		.pgfailback    = -FAILBACK_IMMEDIATE,
-		.prio_name     = PRIO_ALUA,
-		.no_path_retry = 10,
-	},
 	/*
 	 * Imation/Nexsan
 	 */
@@ -1048,8 +1122,33 @@ static struct hwentry default_hw[] = {
 		.no_path_retry = 30,
 	},
 	/*
-	 * Xiotech
+	 * Violin Systems
 	 */
+	{
+		/* 3000 / 6000 Series */
+		.vendor        = "VIOLIN",
+		.product       = "SAN ARRAY$",
+		.pgpolicy      = GROUP_BY_SERIAL,
+		.no_path_retry = 30,
+	},
+	{
+		/* 3000 / 6000 Series - ALUA mode */
+		.vendor        = "VIOLIN",
+		.product       = "SAN ARRAY ALUA",
+		.hwhandler     = "1 alua",
+		.pgpolicy      = GROUP_BY_PRIO,
+		.pgfailback    = -FAILBACK_IMMEDIATE,
+		.prio_name     = PRIO_ALUA,
+		.no_path_retry = 30,
+	},
+	{
+		/* FSP 7000 family */
+		.vendor        = "VIOLIN",
+		.product       = "CONCERTO ARRAY",
+		.pgpolicy      = MULTIBUS,
+		.no_path_retry = 30,
+	},
+		/* Xiotech */
 	{
 		/* Intelligent Storage Elements family */
 		.vendor        = "(XIOTECH|XIOtech)",
@@ -1071,36 +1170,11 @@ static struct hwentry default_hw[] = {
 		.pgpolicy      = MULTIBUS,
 		.no_path_retry = 30,
 	},
-	/*
-	 * Violin Memory
-	 */
-	{
-		/* 3000 / 6000 Series */
-		.vendor        = "VIOLIN",
-		.product       = "SAN ARRAY$",
-		.pgpolicy      = GROUP_BY_SERIAL,
-		.no_path_retry = 30,
-	},
-	{
-		.vendor        = "VIOLIN",
-		.product       = "SAN ARRAY ALUA",
-		.hwhandler     = "1 alua",
-		.pgpolicy      = GROUP_BY_PRIO,
-		.pgfailback    = -FAILBACK_IMMEDIATE,
-		.prio_name     = PRIO_ALUA,
-		.no_path_retry = 30,
-	},
-	{
-		/* FSP 7000 family */
-		.vendor        = "VIOLIN",
-		.product       = "CONCERTO ARRAY",
-		.pgpolicy      = MULTIBUS,
-		.no_path_retry = 30,
-	},
 	/*
 	 * Promise Technology
 	 */
 	{
+		/* VTrak family */
 		.vendor        = "Promise",
 		.product       = "VTrak",
 		.bl_product    = "VTrak V-LUN",
@@ -1111,6 +1185,7 @@ static struct hwentry default_hw[] = {
 		.no_path_retry = 30,
 	},
 	{
+		/* Vess family */
 		.vendor        = "Promise",
 		.product       = "Vess",
 		.bl_product    = "Vess V-LUN",
@@ -1133,17 +1208,7 @@ static struct hwentry default_hw[] = {
 		.no_path_retry = 30,
 	},
 	/*
-	 * Generic NVMe devices
-	 */
-	{
-		.vendor        = "NVME",
-		.product       = ".*",
-		.uid_attribute = "ID_WWN",
-		.checker_name  = NONE,
-		.retain_hwhandler = RETAIN_HWHANDLER_OFF,
-	},
-	/*
-	 * Dot Hill Systems - Seagate Technology
+	 * Seagate Technology (Dot Hill Systems)
 	 */
 	{
 		/* SANnet family */
@@ -1180,10 +1245,22 @@ static struct hwentry default_hw[] = {
 		.pgpolicy      = MULTIBUS,
 		.no_path_retry = 30,
 	},
+	/*
+	 * INSPUR
+	 */
+	{
+		/* AS5300/AS5500 G2 */
+		.vendor        = "INSPUR",
+		.product       = "MCS",
+		.pgpolicy      = GROUP_BY_PRIO,
+		.pgfailback    = -FAILBACK_IMMEDIATE,
+		.prio_name     = PRIO_ALUA,
+	},
 	/*
 	 * EOL
 	 */
 	{
+		/* NULL */
 		.vendor        = NULL,
 		.product       = NULL,
 	},
