@@ -3,9 +3,9 @@
 #include <sys/time.h>
 #include <time.h>
 
-#include <prio.h>
+#include "prio.h"
 
-int getprio (struct path * pp, char * args)
+int getprio(struct path * pp, char * args, unsigned int timeout)
 {
 	struct timeval tv;
 
