@@ -104,7 +104,7 @@ get_word (char * sentence, char ** word)
 	}
 	strncpy(*word, sentence, len);
 	strchop(*word);
-	condlog(4, "*word = %s, len = %i", *word, len);
+	condlog(5, "*word = %s, len = %i", *word, len);
 
 	if (*p == '\0')
 		return 0;
@@ -176,6 +176,7 @@ int devt2devname(char *devname, int devname_len, char *devt)
 	if (stat("/sys/dev/block", &statbuf) == 0) {
 		/* Newer kernels have /sys/dev/block */
 		sprintf(block_path,"/sys/dev/block/%u:%u", major, minor);
+		dev[FILE_NAME_SIZE - 1] = '\0';
 		if (lstat(block_path, &statbuf) == 0) {
 			if (S_ISLNK(statbuf.st_mode) &&
 			    readlink(block_path, dev, FILE_NAME_SIZE-1) > 0) {
@@ -187,11 +188,12 @@ int devt2devname(char *devname, int devname_len, char *devt)
 					return 1;
 				}
 				p++;
-				strncpy(devname, p, devname_len);
+				strlcpy(devname, p, devname_len);
 				return 0;
 			}
 		}
-		goto skip_proc;
+		condlog(4, "%s is invalid", block_path);
+		return 1;
 	}
 	memset(block_path, 0, sizeof(block_path));
 
@@ -220,7 +222,7 @@ int devt2devname(char *devname, int devname_len, char *devt)
 		}
 	}
 	fclose(fd);
-skip_proc:
+
 	if (strncmp(block_path,"/sys/block", 10)) {
 		condlog(3, "No device found for %u:%u", major, minor);
 		return 1;
@@ -271,48 +273,6 @@ dev_t parse_devt(const char *dev_t)
 	return makedev(maj, min);
 }
 
-char *parse_uid_attribute_by_attrs(char *uid_attrs, char *path_dev)
-{
-	char *uid_attribute;
-	char *uid_attr_record;
-	char *dev;
-	char *attr;
-	char *tmp;
-	int  count;
-
-	if(!uid_attrs || !path_dev)
-		return NULL;
-
-	count = get_word(uid_attrs, &uid_attr_record);
-	while (uid_attr_record) {
-		tmp = strrchr(uid_attr_record, ':');
-		if (!tmp) {
-			free(uid_attr_record);
-			if (!count)
-				break;
-			uid_attrs += count;
-			count = get_word(uid_attrs, &uid_attr_record);
-			continue;
-		}
-		dev = uid_attr_record;
-		attr = tmp + 1;
-		*tmp = '\0';
-
-		if(!strncmp(path_dev, dev, strlen(dev))) {
-			uid_attribute = STRDUP(attr);
-			free(uid_attr_record);
-			return uid_attribute;
-		}
-
-		free(uid_attr_record);
-		if (!count)
-			break;
-		uid_attrs += count;
-		count = get_word(uid_attrs, &uid_attr_record);
-	}
-	return NULL;
-}
-
 void
 setup_thread_attr(pthread_attr_t *attr, size_t stacksize, int detached)
 {
@@ -505,3 +465,8 @@ void free_scandir_result(struct scandir_result *res)
 		FREE(res->di[i]);
 	FREE(res->di);
 }
+
+void close_fd(void *arg)
+{
+	close((long)arg);
+}
